import numpy as np
import xarray as xr
import matplotlib.pyplot as plt
import geopandas as gpd
import pandas as pd
from shapely.geometry import Point, Polygon
import sys
import pickle
import glob

#download climate variables from nc file
def bring(name):
	with xr.open_dataset(str(name)) as data: 
		values=data.tas.values
		lon=data.lon.values
		lat=data.lat.values
		dates=data.time.values
		values=np.swapaxes(values,1,2)
	return(values,lon,lat,dates)

#find the indices in longitude and latitude and weightings of grid cells that fall within a regions geometry
def region_lump(poly,lon,lat):

	#find the minimum and maximum bounds of the country within which to search for grid cells that actually fall within the region
	minx=poly.bounds[0]
	maxx=poly.bounds[2]
	miny=poly.bounds[1]
	maxy=poly.bounds[3]
	
	#extract the indices of lon/lat which are within the min max bounds specified above
	lon_i=np.where((lon>minx) & (lon<maxx) == True)
	lat_i=np.where((lat>miny) & (lat<maxy) == True)
	
	#empty list to store rain values for the country
	lons=[]
	lats=[]
	weights=[]

	#for states/countries who straddle the 180'E line, sack it off
	if not maxx-minx>300:
		
		#search the min/max bounds for grid cells that are in the country, then append the variable from these grid cells to the empty list
		for i in range(len(lon_i[0][:])):
			for j in range(len(lat_i[0][:])):	
				#counts the number of scatter points within the grid cell that fall in the region
				count=0.0
				#the resolution of our scatter points
				res=0.5/10
				for z in range(10):
					for y in range(10):
						p1=Point(lon[lon_i[0][i]]+res*(z-5.5),lat[lat_i[0][j]]+res*(y-5.5))	
						if poly.contains(p1):
							count=count+1			
				if count!=0:
					#print("NEW!!")
					lons.append(lon_i[0][i])
					lats.append(lat_i[0][j])
					weights.append(count/100)
	else:
		print("that was shit")				
	return(lons,lats,weights)

#find two means, the monthly means per year, and the monthly means over the whole time period
#arguments are the variable, var, 2D (one space, one time), the percentile, percent, of which you want to calculate, and the dates 
def month_means(var,dates):
        spatials=len(var[0,:])
        means1=np.zeros((12,spatials))
        years=len(np.unique(dates.year))
        means2=np.zeros((years,12,spatials))
        counts2=np.zeros((years,12))
        year0=dates[0].year
        for i in range(len(dates)):
                year=dates[i].year-year0
                month=dates[i].month-1
                means2[year,month,:]=means2[year,month,:]+var[i,:]
                counts2[year,month]=counts2[year,month]+1
        for i in range(years):
                for j in range(12):
                        means2[i,j,:]=means2[i,j,:]/counts2[i,j]
        means1=np.mean(means2,axis=0)
        return(means1,means2)

#find monthly thresholds at the given percentile
def month_thresh(var,percent,dates):
        thresholds=np.zeros((12,len(var[0,:])))
        for i in range(12):
                index=np.where((dates.month==i+1))
                things=(np.squeeze(var[index,:]))
                thresholds[i,:]=np.squeeze(np.percentile(things,percent,axis=0))
        return(thresholds)

#define experiment and variable
experiment="T.5"
variable='tas'

#download climate data
conversion=86486.5
abs0=273.15

#set file path for climate data
file_path='/p/projects/impactee/ERA5/'

#importing and plotting country maps
mask=gpd.read_file('gadm36_levels.gpkg',layer=1)

#extract country geometry
#input 3-letter country iso to specify calculation
c_name=str(sys.argv[1])
index=np.where(mask.GID_0==c_name)
poly=mask.geometry[index[0]]
regions_n=mask.NAME_1[index[0]]
no_regions=len(index[0][:])

print("extracted country geometry")

#for reference period
decades1=[1979,1981,1991,2001,2011]
decades2=[1980,1990,2000,2010,2018]

#empty lists to hold arrays for each subregion of the country
lon_c=[]
lat_c=[]
weights_c=[]
clim=[]

for i in range(len(decades1)):

	print("loading decade " + str(decades1[i]))

	#load that decade of climate data	
	file_name=file_path + variable + '/' + variable + "_day_ERA5-0p5deg_" + str(decades1[i]) + "0101-" + str(decades2[i]) + "1231.nc4"

	#if the first decade, find the coordinates and weightings of each subregion 
	if i==0:	
		[cl,lon,lat,dts]=bring(file_name)
		dates=dts
		file_save='coords/real/lon_lat_weights_T.5_' + c_name + '.pkl'
		file_list=glob.glob(file_save)
		if len(file_list)==0:
			for x in range(no_regions):
				[lon_,lat_,weights_]=region_lump(poly[index[0][x]],lon,lat)
				lon_c.append(lon_)
				lat_c.append(lat_)
				weights_c.append(weights_)
				clim.append(cl[:,lon_,lat_])
				print("found region " + str(x) + " of " + str(no_regions))
	
			with open(file_save,'wb') as f:
				pickle.dump([lon_c,lat_c,weights_c],f)		
		else:
			print('loading saved coords!') 
			with open(file_save,'rb') as f:
				[lon_c,lat_c,weights_c]=pickle.load(f)
			for x in range(no_regions):
				clim.append(cl[:,lon_c[x],lat_c[x]])
	
	#for other decadess, append the extra climate data to each subregion 
	else:		
		[cl,lon,lat,dts]=bring(file_name)
		for x in range(no_regions):
			clim[x]=np.concatenate((clim[x],cl[:,lon_c[x],lat_c[x]]),axis=0)
		dates=np.concatenate((dates,dts),axis=0)


#convert from kelvin to degrees centigrade
for i in range(no_regions):
	clim[i]=clim[i]-abs0

dates=pd.DatetimeIndex(dates)

years=decades2[-1]-decades1[0]+1

#calculate measures
measure=np.zeros((3,no_regions,years,12))

for x in range(no_regions):
	if len(lon_c[x])==0:
		measure[:,x,:,:]='NaN'
	else:

		#calculate grid cell climatology and monthly mean
		[means1,means2]=month_means(clim[x],dates)
	
		#define grid cell specific monthly measure	
		no_gr_cells=len(clim[x][0,:])
		T_hat=np.zeros((2,no_gr_cells,years,12))

		print("calculating grid cell specific monthly variations for region " + str(x) + " of " + str(no_regions))

		#sum over grid cells:
		for i in range(no_gr_cells):
			
			#monthly mean temp, summed to regional level and weighted by grid cell weights
			measure[0,x,:,:]=measure[0,x,:,:]+means2[:,:,i]*weights_c[x][i]	
			counts=np.zeros((years,12))
			#sum over the dates to the monthly level
			for t in range(len(dates)):
				month=dates.month[t]-1
				year=dates.year[t]-dates.year[0]	
								
				#monthly grid cell variation from the monthly climatology
				T_hat[0,i,year,month]=T_hat[0,i,year,month]+((clim[x][t,i]-means1[month,i])**2)
		
				#monthly grid cell variation from the monthly mean
				T_hat[1,i,year,month]=T_hat[1,i,year,month]+((clim[x][t,i]-means2[year,month,i])**2)
			
				#count the number of days per month for each year
				counts[year,month]=counts[year,month]+1	

		#square root and then divide by number of days per month to get grid cell specific monthly variation 	
		T_hat=T_hat/counts	
		T_hat=np.sqrt(T_hat)

		print("aggregating to the regional level")

		#aggregate to the regional level, using the weightings
		for z in range(2):
			for i in range(years):
				for j in range(12):
					month=dates.month[t]-1
					year=dates.year[t]-dates.year[0]
					measure[z+1,x,i,j]=np.sum(T_hat[z,:,i,j]*weights_c[x][:])

		#normalise by the sum of the weights of that region
		measure[:,x,:,:]=measure[:,x,:,:]/np.sum(weights_c[x][:])		

#save
np.save("measures/temp/" + experiment + "_" + c_name + "_measure.npy",measure)
	











